@@ -15,7 +15,7 @@
 #include <sstream>
 #include <string>
 
-#include "../include/unordered_dense.h"
+#include "ankerl_unordered_dense.h"
 #include "utils.h"
 
 #include "htslib/faidx.h"
@@ -169,13 +169,21 @@ namespace Utils {
             start = end + 1;
             end = s.find(delim, start);
             r->end = std::stoi(s.substr(start, end - start));
+            if (end != std::string::npos) {
+                start = end + 1;
+                end = s.find(delim, start);
+                r->markerPos = std::stoi(s.substr(start, end - start));
+                start = end + 1;
+                end = s.find(delim, start);
+                r->markerPosEnd = std::stoi(s.substr(start, end - start));
+            }
         } else {
             r->start = std::stoi(s.substr(start, s.size()));
             r->end = r->start + 1;
         }
     }
 
-    Region parseRegion(std::string &s) {
+    EXPORT Region parseRegion(std::string &s) {
         Region reg;
         std::string s2;
         if (s.find(":") != std::string::npos) {
@@ -186,8 +194,6 @@ namespace Utils {
             Utils::strToRegion(&reg, s, ',');
         } else if (s.find("\t") != std::string::npos) {
             Utils::strToRegion(&reg, s, '\t');
-        } else if (s.find("_") != std::string::npos) {
-            Utils::strToRegion(&reg, s, '_');
         } else if (s.find(" ") != std::string::npos) {
             Utils::strToRegion(&reg, s, ' ');
         } else {
@@ -201,10 +207,14 @@ namespace Utils {
         if (reg.start == reg.end) {
             reg.end += 1;
         }
-        reg.markerPos = -1;
         return reg;
     }
 
+    std::string Region::toString() {
+        return chrom + ":" + std::to_string(start) + "-" + std::to_string(end)
+           + ((markerPos >= 0) ? ":" + std::to_string(markerPos) + ":" + std::to_string(markerPosEnd) : "");
+    }
+
     std::filesystem::path makeFilenameFromRegions(std::vector<Utils::Region> &regions) {
         std::filesystem::path fname = "GW~";
         for (auto &rgn: regions) {
@@ -363,20 +373,26 @@ namespace Utils {
     }
 
 
-    Dims parseDimensions(std::string &s) {
+    EXPORT Dims parseDimensions(std::string &s) {
         Dims d = {0, 0};
         int start = 0;
         int end = (int)s.find('x');
         if (end == (int)s.size()) {
-            throw std::runtime_error("Error: 'x' not in dimensions");
+            end = (int)s.find(',');
+            if (end == (int)s.size()) {
+                throw std::runtime_error("Error: 'x' or ',' not in dimensions");
+            }
         }
         d.x = std::stoi(s.substr(start, end - start));
         start = end + 1;
         end = (int)s.find('x', start);
-        d.y = std::stoi(s.substr(start, end - start));
-        if (d.x == 0) {
-            throw std::runtime_error("Error: dimension x was 0");
+        if (end == (int)s.size()) {
+            end = (int)s.find(',');
         }
+        d.y = std::stoi(s.substr(start, end - start));
+//        if (d.x == 0) {
+//            throw std::runtime_error("Error: dimension x was 0");
+//        }
         return d;
     }
 
